'use strict';

const Promise = require('bluebird');
const Errors = require('common-errors');
const getInternalData = require('../utils/getInternalData.js');
const isActive = require('../utils/isActive.js');
const isBanned = require('../utils/isBanned.js');
const key = require('../utils/key.js');

var _require = require('../constants.js');

const USERS_DATA = _require.USERS_DATA;
const USERS_METADATA = _require.USERS_METADATA;
const USERS_PUBLIC_INDEX = _require.USERS_PUBLIC_INDEX;
const USERS_ALIAS_TO_LOGIN = _require.USERS_ALIAS_TO_LOGIN;
const USERS_ALIAS_FIELD = _require.USERS_ALIAS_FIELD;


module.exports = function assignAlias(opts) {
  const redis = this.redis;
  const defaultAudience = this.config.jwt.defaultAudience;
  const username = opts.username;
  const alias = opts.alias;


  return Promise.bind(this, username).then(getInternalData).tap(isActive).tap(isBanned).then(data => {
    if (data[USERS_ALIAS_FIELD]) {
      throw new Errors.HttpStatusError(417, 'alias is already assigned');
    }

    return redis.hsetnx(USERS_ALIAS_TO_LOGIN, alias, username);
  }).then(assigned => {
    if (assigned === 0) {
      throw new Errors.HttpStatusError(409, 'alias was already taken');
    }

    return redis.pipeline().sadd(USERS_PUBLIC_INDEX, username).hset(key(username, USERS_DATA), USERS_ALIAS_FIELD, alias).hset(key(username, USERS_METADATA, defaultAudience), USERS_ALIAS_FIELD, JSON.stringify(alias)).exec();
  });
};