'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

const Promise = require('bluebird');
const mapValues = require('lodash/mapValues');
const stringify = JSON.stringify.bind(JSON);

const redisKey = require('../utils/key.js');
const userExists = require('../utils/userExists.js');

var _require = require('../constants.js');

const USERS_DATA = _require.USERS_DATA;
const USERS_METADATA = _require.USERS_METADATA;
const USERS_BANNED_FLAG = _require.USERS_BANNED_FLAG;
const USERS_TOKENS = _require.USERS_TOKENS;
const USERS_BANNED_DATA = _require.USERS_BANNED_DATA;


function lockUser(_ref) {
  let username = _ref.username;
  let reason = _ref.reason;
  let whom = _ref.whom;
  let remoteip = _ref.remoteip;
  const redis = this.redis;
  const config = this.config;
  const defaultAudience = config.jwt.defaultAudience;

  const data = {
    banned: true,
    [USERS_BANNED_DATA]: {
      reason: reason || '',
      whom: whom || '',
      remoteip: remoteip || ''
    }
  };

  return redis.pipeline().hset(redisKey(username, USERS_DATA), USERS_BANNED_FLAG, 'true')
  // set .banned on metadata for filtering & sorting users by that field
  .hmset(redisKey(username, USERS_METADATA, defaultAudience), mapValues(data, stringify)).del(redisKey(username, USERS_TOKENS)).exec();
}

function unlockUser(_ref2) {
  let username = _ref2.username;
  const redis = this.redis;
  const config = this.config;
  const defaultAudience = config.jwt.defaultAudience;


  return redis.pipeline().hdel(redisKey(username, USERS_DATA), USERS_BANNED_FLAG)
  // remove .banned on metadata for filtering & sorting users by that field
  .hdel(redisKey(username, USERS_METADATA, defaultAudience), 'banned', USERS_BANNED_DATA).exec();
}

/**
 * Bans/unbans existing user
 * @param  {Object} opts
 * @return {Promise}
 */
module.exports = function banUser(opts) {
  return Promise.bind(this, opts.username).then(userExists).then(username => _extends({}, opts, { username: username })).then(opts.ban ? lockUser : unlockUser);
};